// --------------------------------------------------------------------------------------
//
// AnalogInput.CS                                                  Ulrich Mller 16.06.05
//                                                                 PCS1700   C# Final SP2
// Sensoren an den Analog-Eingngen
//
// --------------------------------------------------------------------------------------

using System;
using System.Threading;

namespace FishDevices40 {
	public class AnalogInput : DeviceBase {

		private int limitLow, limitHigh;
		private int actValue, oldRange;

		public delegate void Limit(object sender, int Value);
		public event         Limit ChangedToLow;
		public event         Limit ChangedToHigh;
		public event         Limit ChangedToNormal;
		public event         Limit ShowActualValue;

		// --- Constructor -------------------------------------------------------------

		public AnalogInput(FishDevices fd, Inp AnalogNumber, int LimitLow, int LimitHigh) 
			: this(fd, true, Convert.ToInt32(AnalogNumber), LimitLow, LimitHigh) {}
		public AnalogInput(FishDevices fd, int AnalogNumber, int LimitLow, int LimitHigh) 
			: this(fd, true, AnalogNumber, LimitLow, LimitHigh) {}
		public AnalogInput(FishDevices fd, bool WithEvents, 
			Inp AnalogNumber, int LimitLow, int LimitHigh) 
			: this(fd, Convert.ToInt32(AnalogNumber), LimitLow, LimitHigh){}
		public AnalogInput(FishDevices fd, bool WithEvents, 
			int AnalogNumber, int LimitLow, int LimitHigh) 
			: base(fd, WithEvents, AnalogNumber) {
			this.limitLow  = LimitLow;
			this.limitHigh = LimitHigh;
			this.oldRange = 0;
			base.threadInterval = 22;
			fd.AddDevice(this);
			if(base.WithEvents) {
				base.deviceThread = new Thread(new ThreadStart(EventLoop));
				base.deviceThread.IsBackground = true;
			}
		}

		// --- AnalogLoop : Scannen des Analog-Einganges und Auslsen von Ereignissen ---

		private void EventLoop() {
			while(!fd.NotHalt && !fd.ESC) {
				actValue = fd.GetAnalogPur(base.DeviceNumber);
				if((uint)actValue == FishBase.rbFehler)
					throw(new FishDevException("InterfaceProblem.AnalogInput"));
				if(actValue < limitLow && oldRange != 1) {
					oldRange = 1;
					if(ChangedToLow != null) ChangedToLow(this, actValue);
				}
				else if(actValue > limitHigh && oldRange != 3) {
					oldRange = 3;
					if(ChangedToHigh != null) ChangedToHigh(this, actValue);
				}
				else if(actValue >= limitLow && actValue <= limitHigh && oldRange != 2) {
					oldRange = 2;
					if(ChangedToNormal != null) ChangedToNormal(this, actValue);
				}
				if(ShowActualValue != null) ShowActualValue(this, actValue);
				Thread.Sleep(base.threadInterval);
			}
		}

		// --- Eigenschaften des Analog-Einganges -----------------------------

		public int AnalogNumber {
			get{return base.DeviceNumber;}
		}
		public int ActualValue {
			get {
				try {return fd.GetAnalog(base.DeviceNumber);}
				catch(FishFaceException eft){
					throw(new FishDevException(eft.Message + ".AnalogInput.ActualValue"));
				}
			}
		}
		public int LimitHigh {
			get{return this.limitHigh;}
			set{limitHigh = value;}
		}
		public int LimitLow {
			get{return this.limitLow;}
			set{limitLow = value;}
		}
	}

	// --- PhotoResistor : Klasse fr den Photowiderstand --------------------

	public class PhotoResistor : AnalogInput{
		public PhotoResistor(FishDevices fd, Inp AnalogNumber, int LimitLow, int LimitHigh)
			: this(fd, Convert.ToInt32(AnalogNumber), LimitLow, LimitHigh) {}
		public PhotoResistor(FishDevices fd, int AnalogNumber, int LimitLow, int LimitHigh) 
			: this(fd, true, AnalogNumber, LimitLow, LimitHigh) {}
		public PhotoResistor(FishDevices fd, bool WithEvents,
			Inp AnalogNumber, int LimitLow, int LimitHigh) 
			: this(fd, WithEvents, Convert.ToInt32(AnalogNumber), LimitLow, LimitHigh) {}
		public PhotoResistor(FishDevices fd, bool WithEvents, 
			int AnalogNumber, int LimitLow, int LimitHigh) 
			: base(fd, WithEvents, AnalogNumber, LimitLow, LimitHigh) {}
	}

	// --- NTC : Klasse fr den NTC (Negative Temmperature Coefficient) ---
	
	public class NTC : AnalogInput{
		public NTC(FishDevices fd, Inp AnalogNumber, int LimitLow, int LimitHigh)
			: this(fd, Convert.ToInt32(AnalogNumber), LimitLow, LimitHigh) {}
		public NTC(FishDevices fd, int AnalogNumber, int LimitLow, int LimitHigh) 
			: this(fd, true, AnalogNumber, LimitLow, LimitHigh) {}
		public NTC(FishDevices fd, bool WithEvents,
			Inp AnalogNumber, int LimitLow, int LimitHigh) 
			: this(fd, WithEvents, Convert.ToInt32(AnalogNumber), LimitLow, LimitHigh) {}
		public NTC(FishDevices fd, bool WithEvents, 
			int AnalogNumber, int LimitLow, int LimitHigh) 
			: base(fd, WithEvents, AnalogNumber, LimitLow, LimitHigh) {}
	}

	// --- PotentioMeter : Klasse fr das Drehpotitiometer -----------------

	public class PotentioMeter : AnalogInput{
		public PotentioMeter(FishDevices fd, Inp AnalogNumber, int LimitLow, int LimitHigh)
			: this(fd, Convert.ToInt32(AnalogNumber), LimitLow, LimitHigh) {}
		public PotentioMeter(FishDevices fd, int AnalogNumber, int LimitLow, int LimitHigh) 
			: this(fd, true, AnalogNumber, LimitLow, LimitHigh) {}
		public PotentioMeter(FishDevices fd, bool WithEvents,
			Inp AnalogNumber, int LimitLow, int LimitHigh) 
			: this(fd, WithEvents, Convert.ToInt32(AnalogNumber), LimitLow, LimitHigh) {}
		public PotentioMeter(FishDevices fd, bool WithEvents, 
			int AnalogNumber, int LimitLow, int LimitHigh) 
			: base(fd, WithEvents, AnalogNumber, LimitLow, LimitHigh) {}
	}
}
