// --------------------------------------------------------------------------------------
//
// BinaryInput.CS                                                  Ulrich Mller 16.06.05
//                                                                 PCS1700   C# Final SP2
// Klassen fr einfache Digital-Eingnge
//
// --------------------------------------------------------------------------------------

using System;
using System.Threading;

namespace FishDevices40 {
	// BinaryInput : Klasse fr den unspezifizierten Zugriff auf 
	// die I-(E-)Eingnge eines Interfaces
	public class BinaryInput : DeviceBase {

		public delegate void Changed(object sender);
		public event         Changed ChangedToTrue;
		public event         Changed ChangedToFalse;

		private int actValue, oldValue;

		// Constructor : Erstellen eine Objektes fr einen I-Eingang an einer
		// whlbaren Interface-Instanz</summary>
		public BinaryInput(FishDevices fd, Inp InputNumber)
			: this(fd, true, Convert.ToInt32(InputNumber)) {}
		public BinaryInput(FishDevices fd, int InputNumber) 
			: this(fd, true, InputNumber) {}
		public BinaryInput(FishDevices fd, bool WithEvents, Inp  InputNumber) 
			: this(fd, WithEvents, Convert.ToInt32(InputNumber)) {}
		public BinaryInput(FishDevices fd, bool WithEvents, int InputNumber) 
			: base(fd, WithEvents, InputNumber) {
			base.threadInterval = 22;
			fd.AddDevice(this);
			if(base.WithEvents) {
				base.deviceThread = new Thread(new ThreadStart(EventLoop));
				base.deviceThread.IsBackground = true;
			}
		}

		// EventLoop : Scannen des I-Einganges und Auslsen von Ereignissen
		private void EventLoop() {
			oldValue = fd.GetInputs() & (1 << (base.DeviceNumber-1));
			while(!fd.NotHalt && !fd.ESC) {
				if((uint)fd.GetInputs() == FishBase.rbFehler) 
					throw(new FishDevException("InterfaceProblem.BinaryInput"));
				actValue = fd.GetInputs() & (1 << (base.DeviceNumber-1));
				if(actValue > oldValue) if(ChangedToTrue  != null) 
											ChangedToTrue(this);
				if(actValue < oldValue) if(ChangedToFalse != null) 
											ChangedToFalse(this);
				oldValue = actValue;
				Thread.Sleep(base.threadInterval);
			}
		}

		// --- Eigenschaften des I-Einganges ----------------------

		public int InputNumber {
			get{return base.DeviceNumber;}
		}

		public bool IsTrue {
			get{
				try{return fd.GetInput(base.DeviceNumber);}
				catch(FishFaceException eft) {
					throw(new FishDevException(eft.Message + ".BinaryInput.InputNumber")); }
			}
		}

		// --- Methoden zum Warten auf Vernderungen des I-Einganges ------------
		
		public void WaitForInput() { this.WaitForInput(true);}
		public void WaitForInput(bool OnOff) {
			try{fd.WaitForInput(InputNumber, OnOff);}
			catch(FishFaceException eft) {
				throw(new FishDevException(eft.Message + ".BinaryInput.WaitForInput")); }
		}

		public void WaitForHigh() {
			try{
				fd.WaitForInput(InputNumber, false);
				fd.WaitForInput(InputNumber, true);
			}
			catch(FishFaceException eft){
				throw(new FishDevException(eft.Message + ".BinaryInput.WaitForHigh")); }
		}

		public void WaitForLow() {
			try{
				fd.WaitForInput(InputNumber, true);
				fd.WaitForInput(InputNumber, false);
			}
			catch(FishFaceException eft){
				throw(new FishDevException(eft.Message + ".BinaryInput.WaitForLow")); }
		}
	}

	// --- Sensor : Klasse fr den Taster ---------------------------------------

	public class Sensor : BinaryInput {
		public Sensor(FishDevices fd, bool WithEvents, Inp InputNumber) 
			: this(fd, WithEvents, Convert.ToInt32(InputNumber)) {}
		public Sensor(FishDevices fd, bool WithEvents, int InputNumber) 
			: base(fd, WithEvents, InputNumber) {}
		public Sensor(FishDevices fd, Inp InputNumber) 
			: this(fd, Convert.ToInt32(InputNumber)) {}
		public Sensor(FishDevices fd, int InputNumber) 
			: base(fd, true, InputNumber) {}
	}

	public class PhotoTransistor : BinaryInput {
		public PhotoTransistor(FishDevices fd, bool WithEvents, Inp InputNumber) 
			: this(fd, WithEvents, Convert.ToInt32(InputNumber)) {}
		public PhotoTransistor(FishDevices fd, bool WithEvents, int InputNumber) 
			: base(fd, WithEvents, InputNumber) {}
		public PhotoTransistor(FishDevices fd, Inp InputNumber) 
			: this(fd, Convert.ToInt32(InputNumber)) {}
		public PhotoTransistor(FishDevices fd, int InputNumber) 
			: base(fd, true, InputNumber) {}
	}

	public class ReedContact : BinaryInput {
		public ReedContact(FishDevices fd, bool WithEvents, Inp InputNumber) 
			: this(fd, WithEvents, Convert.ToInt32(InputNumber)) {}
		public ReedContact(FishDevices fd, bool WithEvents, int InputNumber) 
			: base(fd, WithEvents, InputNumber) {}
		public ReedContact(FishDevices fd, Inp InputNumber) 
			: this(fd, Convert.ToInt32(InputNumber)) {}
		public ReedContact(FishDevices fd, int InputNumber) 
			: base(fd, true, InputNumber) {}
	}
}
