// --------------------------------------------------------------------------------------
//
// DualOutput.CS                                                   Ulrich Mller 16.06.05
//                                                                 PCS1700   C# Final SP2
// Devices mit Anschlu an beide Pole eines M-Ausganges
//
// --------------------------------------------------------------------------------------

using System;
using System.Threading;

namespace FishDevices40
{
	public class DualOutput	: DeviceBase {
		
		protected int stateValue;

		// --- Constructor ------------------------------------------

		public DualOutput(FishDevices fd, Out OutputNumber) 
			: this(fd, Convert.ToInt32(OutputNumber)) {}
		public DualOutput(FishDevices fd, int OutputNumber) 
			: base(fd, false, OutputNumber) {
			fd.AddDevice(this);
			this.stateValue = 0;
		}

		// --- Eigenschaften --------------------------------

		public int State {
			get{return stateValue;}
		}
		public int OutputNumber {
			get{return base.DeviceNumber;}
		}

		// --- Methoden --------------------------------------------------

		public void On()            {this.Left(15);}
		public void On(Speed speed) {this.Left(speed);}
		public void On(int Speed)   {this.Left(Speed);}

		public void Left() {this.Left(15);}
		public void Left(Speed speed) {this.Left(Convert.ToInt32(speed));}
		public void Left(int Speed) {
			stateValue = 1;
			try{
				fd.SetMotor(base.DeviceNumber, stateValue, Speed);
			}
			catch(FishFaceException eft) {
				throw(new FishDevException(eft.Message + ".DualOutput.Left"));}
		}

		public void Right() {this.Right(15);}
		public void Right(Speed speed) {this.Right(Convert.ToInt32(speed));}
		public void Right(int Speed) {
			stateValue = 2;
			try{
				fd.SetMotor(base.DeviceNumber, stateValue, Speed);
			}
			catch(FishFaceException eft) {
				throw(new FishDevException(eft.Message + ".DualOutput.Right"));}
		}

		public void Off() {
			stateValue = 0;
			try{
				fd.SetMotor(base.DeviceNumber, stateValue);
			}
			catch(FishFaceException eft){
				throw(new FishDevException(eft.Message + ".DualOutput.Off"));}
		}

		public void Go(Dir dir) {this.Go(Convert.ToInt32(dir));}
		public void Go(int Dir) {this.Go(Dir, 15);}
		public void Go(Dir dir, Speed speed) {
			this.Go(Convert.ToInt32(dir), Convert.ToInt32(speed));}
		public void Go(int Dir, int Speed) {
			stateValue = Dir;
			try{
				fd.SetMotor(base.DeviceNumber, stateValue, Speed);
			}
			catch(FishFaceException eft){
				throw(new FishDevException(eft.Message + ".DualOutput.Go"));}
		}
	}

	// --- Motor : Klasse fr einen Motor am M-Ausgang ---------------------------

	public class Motor : DualOutput {
		public Motor(FishDevices fd, Out MotorNumber) 
			: this(fd, Convert.ToInt32(MotorNumber)) {}
		public Motor(FishDevices fd, int MotorNumber) : base(fd, MotorNumber) {}

		public void Forward() {base.Left();}
		public void Forward(Speed speed) {this.Left(speed);}
		public void Forward(int Speed) {this.Left(Speed);}

		public void Backward() {base.Right();}
		public void Backward(Speed speed) {this.Right(speed);}
		public void Backward(int Speed) {this.Right(Speed);}

		public void Up() {base.Left();}
		public void Up(Speed speed) {this.Left(speed);}
		public void Up(int Speed) {this.Left(Speed);}

		public void Down() {base.Right();}
		public void Down(Speed speed) {this.Right(speed);}
		public void Down(int Speed) {this.Right(Speed);}
		
	}

	// --- DLamp : Klasse fr eine zweipolig angeschlossene Lampe -----------------
	
	public class DLamp : DualOutput {
		
		private int  freqOn, freqOff;
		private bool blinkingOn;
		private Thread BlinkingThread;

		public DLamp(FishDevices fd, Out OutputNumber) 
			: this(fd, Convert.ToInt32(OutputNumber)) {}
		public DLamp(FishDevices fd, int OutputNumber) : base(fd, OutputNumber) {}

		// --- Methoden -------------------

		public void BlinkingOn(int Freq) {
			this.BlinkingOn(Freq, Freq);
		}
		public void BlinkingOn(int FreqOn, int FreqOff) {
			this.freqOn  = FreqOn  > 11 ? FreqOn  : 11;
			this.freqOff = FreqOff > 11 ? FreqOff : 11;
			BlinkingThread = new Thread(new ThreadStart(BlinkingLoop));
			BlinkingThread.IsBackground = true;
			blinkingOn = true;
			BlinkingThread.Start();
		}
		public void BlinkingOff() {
			blinkingOn = false;
		}
		private void BlinkingLoop() {
			bool FlipFlop = true;
			int  BlinkFreq = freqOn;
			while(!fd.NotHalt && !fd.ESC && blinkingOn) {
				if(FlipFlop) {
					this.On();
					FlipFlop  = false;
					BlinkFreq = freqOn;
				}
				else {
					this.Off();
					FlipFlop  = true;
					BlinkFreq = freqOff;
				}
				Thread.Sleep(BlinkFreq);
			}
			this.Off();
		}
	}

	// --- DMagnet : Klasse fr einen zweipolig angeschlossenen Magneten -----------
	
	public class DMagnet : DualOutput {
		public DMagnet(FishDevices fd, Out OutputNumber) 
			: this(fd, Convert.ToInt32(OutputNumber)) {}
		public DMagnet(FishDevices fd, int OutputNumber) : base(fd, OutputNumber) {}
	}

	// --- DPneuValve : Klasse fr ein zweipolig angeschlossenes Magnetventil ------
	
	public class DPneuValve : DualOutput {
		public DPneuValve(FishDevices fd, Out OutputNumber) 
			: this(fd, Convert.ToInt32(OutputNumber)) {}
		public DPneuValve(FishDevices fd, int OutputNumber) : base(fd, OutputNumber) {}
	}

	// --- Buzzer : Klasse fr eine Summer ------------------------------------------

	public class Buzzer : DualOutput {
		public Buzzer(FishDevices fd, Out OutputNumber) 
			: this(fd, Convert.ToInt32(OutputNumber)) {}
		public Buzzer(FishDevices fd, int OutputNumber) : base(fd, OutputNumber) {}
	}
}
