// --------------------------------------------------------------------------------------
//
// FishBa40.CS                                                     Ulrich Mller 16.06.05
//                                                                 PCS1700   C# Final SP2
//                                                                 Version         4.1.99
// Basisklassen fr den Zugriff auf die fischertechnik Interfaces
//              der ROBO-Serie und das Intelligent Interface
//              Reduzierte Version der Assembly FishFace40 fr den Einsatz mit FishDev40
//              Basis : umFish40.DLL
//
// Klasse FishFace : Kapselung und Erweiterung der Funktionen von umFish40.DLL
// Klasse FishFaceException : throw FishFace Ausnahmen
//
// --------------------------------------------------------------------------------------

using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace FishDevices40
{
	public enum IFTypen {
		ftROBO_first_USB       = 0,
		ftIntelligent_IF       = 10,
		ftIntelligent_IF_Slave = 20,
		ftROBO_IF_IIM          = 50,
		ftROBO_IF_USB          = 60,
		ftROBO_IF_COM          = 70,
		rfROBO_IF_Over_RF      = 80,
		ftROBO_IO_Extension    = 90,
		ftROBO_RF_Datalink     = 110
	}

	public enum Port {COM1 = 1, COM2, COM3, COM4}

	public enum Dir {
		Aus    = 0, Off   = Aus,
		Ein    = 1, On    = Ein,
		Links  = 1, Left  = Links,
		Rechts = 2, Right = Rechts
	}

	public enum Inp {
		AX = 1, AY, AXS1, AXS2, AXS3,
		A1 = 1, A2, AV, AZ,
		D1 = 1, D2,
		I1 = 1, I2, I3, I4, I5, I6, I7, I8, I9, I10, I11, I12, I13, I14, I15, I16,
		I17, I18, I19, I20, I21, I22, I23, I24, I25, I26, I27, I28, I29, I30, I31, I32
	}
	public enum IRCode {All = 0, Code1, Code2}
	public enum IRKeys {M3R = 1, M3L, M1, M2, M3, Code2, M1BW, M1FW, M2L, M2R, Code1
	}
	public enum Out {
		M1 = 1, M2, M3, M4, M5, M6, M7, M8, M9, M10, M11, M12, M13, M14, M15, M16,
		O1 = 1, O2, O3, O4, O5, O6, O7, O8, O9, O10, O11, O12, O13, O14, O15, O16,
		O17, O18, O19, O20, O21, O22, O23, O24, O25, O26, O27, O28, O29, O30, O31, O32
}

	public enum Speed {
		Off = 0, L1, L2, L3, Half, L5, L6, Full
	}
	public enum Wait {
		Ende = 0, Time, NotHalt, ESC
	}

	public struct DeviceData {
		public string Name;
		public int    Type;
		public int    SerialNr;
		public string Firmware;
	}

	public class FishFaceException: ApplicationException {
		public FishFaceException() {}
		public FishFaceException(string message) : base(message) {}
		public FishFaceException(string message, Exception inner) : base(message, inner){}
	}

	// ---------------------------------------------------------------------------------
	// Klasse FishBase
	// ---------------------------------------------------------------------------------

	public class FishBase {

		// --- umFish40.DLL Deklarationen ----------------------------------------
		[DllImport("umFish40.DLL")]
		private static extern uint rbOpenInterfaceUSB(int ifTyp, int SerialNr);
		[DllImport("umFish40.DLL")]
		private static extern uint rbOpenInterfaceCOM(int ifTyp, 
			int ComNr, int AnalogZyklen);
		[DllImport("umFish40.DLL")]
		private static extern uint rbCloseInterface(uint iHandle);
		[DllImport("umFish40.DLL")]
		private static extern int rbGetInput(uint iHandle, int InputNr);
		[DllImport("umFish40.DLL")]
		private static extern int rbGetInputs(uint iHandle);
		[DllImport("umFish40.DLL")]
		private static extern int rbGetAnalog(uint iHandle, int AnalogNr);
		[DllImport("umFish40.DLL")]
		private static extern int rbGetIRKey(uint iHandle, int Code, int KeyNr);
		[DllImport("umFish40.DLL")]
		private static extern int rbGetVoltage(uint iHandle, int VoltNr);
		[DllImport("umFish40.DLL")]
		private static extern uint rbSetMotor(uint iHandle, int MotNr, int Dir);
		[DllImport("umFish40.DLL")]
		private static extern uint rbSetMotorEx(uint iHandle, 
			int MotNr, int Dir, int Speed);
		[DllImport("umFish40.DLL")]
		private static extern int rbGetMotors(uint iHandle);
		[DllImport("umFish40.DLL")]
		private static extern uint rbSetMotors(uint iHandle, int MotorStatus);
		[DllImport("umFish40.DLL")]
		private static extern uint rbSetMotorsEx(uint iHandle, int MotorStatus,
			int SpeedStatus, int SpeedStatus16);
		[DllImport("umFish40.DLL")]
		private static extern int rbGetModeStatus(uint iHandle, int MotNr);
		[DllImport("umFish40.DLL")]
		private static extern uint rbSetModeStatus(uint iHandle, int MotNr, int Mode);
		[DllImport("umFish40.DLL")]
		private static extern uint rbSetLamp(uint iHandle, int LampNr, int OnOff);
		[DllImport("umFish40.DLL")]
		private static extern uint rbSetLampEx(uint iHandle, 
			int LampNr, int OnOff, int Power);
		[DllImport("umFish40.DLL")]
		private static extern uint rbRobMotor(uint iHandle, 
			int MotNr, int Dir, int Speed, int ICount);
		[DllImport("umFish40.DLL")]
		private static extern uint rbRobMotors(uint iHandle, 
			int MotorStatus, uint SpeedStatus, int SpeedStatus16, int ModeStatus);
		[DllImport("umFish40.DLL")]
		private static extern int rbGetCounter(uint iHandle, int CounterNr);
		[DllImport("umFish40.DLL")]
		private static extern uint rbSetCounter(uint iHandle, int CounterNr, int ICount);
		[DllImport("umFish40.DLL")]
		private static extern uint rbClearCounters(uint iHandle);
		[DllImport("umFish40.DLL")]
		private static extern uint rbGetActDeviceType(uint iHandle);
		[DllImport("umFish40.DLL")]
		private static extern uint rbGetActDeviceSerialNr(uint iHandle);
		[DllImport("umFish40.DLL")]
		private static extern uint rbGetActDeviceFirmwareNr(uint iHandle);

		// --- Win API Deklarationen ----------------------------------------------------

		[DllImport("kernel32")]
		protected static extern void Sleep(uint mSek);
		[DllImport("kernel32")]
		protected static extern uint GetTickCount();
		[DllImport("user32")]
		protected static extern uint GetAsyncKeyState(uint vKey);
		protected const uint ESCAPE = 0x1B;

		// --- private (globale) Variable ----------------------------------------

		private uint   iHandle;        // --- Handle auf aktive umFish40-Instanz
		private bool   FOpen;          // --- OpenInterface erfolgreich
		private bool   FNotHalt;       // --- NotHalt
		private bool   FDoEvents;      // --- mit/ohne Application.DoEvents

		public const uint rbFehler = 0xE0000001;
		protected const uint rbPollInterval = 10;

		// --- FishFace : Constructor ---------------------------------------------

		public FishBase() {}

		// --- ffentliche/Internal Methoden und Eigenschaften --------------------
		
		// --- ClearCounter : Lschen eines Counters ------------------------------

		internal void ClearCounter(int InputNr) {
			if(!FOpen) throw(new FishFaceException("KeinOpen.ClearCounter"));
			if(rbSetCounter(iHandle, InputNr, 0) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.ClearCounter"));
		}

		// --- ClearCounters : Lschen aller Counter -----------------------------

		internal void ClearCounters() {
			if(!FOpen) throw(new FishFaceException("KeinOpen.ClearCounters"));
			if(rbClearCounters(iHandle) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.ClearCounters"));
		}

		// --- ClearMotors : Lschen aller M-Ausgnge ----------------------------

		internal void ClearMotors() {
			if(!FOpen) throw(new FishFaceException("KeinOpen.ClearMotors"));
			if(rbSetMotors(iHandle, 0) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.ClearMotors"));
		}
		// --- CloseInterface : Schlieen des Interface --------------------------

		internal void CloseInterface() {
			if(FOpen) if(rbCloseInterface(iHandle) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.CloseInterface"));
			FOpen = false;
		}

		// --- ESC : Escape-Taste wird gedrckt -----------------------------
		public bool ESC {
			get {return (GetAsyncKeyState(ESCAPE) != 0);}
		}

		// --- Finish : Feststellen Endewunsch (NotHalt, Escape, InputNr -----

		public bool Finish() {
			return Finish(0);
		}
		public bool Finish(Inp InputNr) {return Finish((int)InputNr);}
		public bool Finish(int InputNr) {
			if(FDoEvents) Application.DoEvents();
			if(!FOpen) throw(new FishFaceException("KeinOpen.Finish"));
			if(FNotHalt || GetAsyncKeyState(ESCAPE) !=0) return true;
			if((uint)GetInputs() == rbFehler) 
				throw(new FishFaceException("InterfaceProblem.Finish"));
			if((InputNr > 0) && GetInput(InputNr)) return true;
			return false;
		}
		public bool Finish(IRCode Code, IRKeys KeyNr) {
			return Finish((int)Code, (int)KeyNr);
		}
		public bool Finish(int Code, int KeyNr) {
			if(FDoEvents) Application.DoEvents();
			if(!FOpen) throw(new FishFaceException("KeinOpen.Finish"));
			if(FNotHalt || GetAsyncKeyState(ESCAPE) !=0) return true;
			if((uint)GetInputs() == rbFehler) 
				throw(new FishFaceException("InterfaceProblem.Finish"));
			if(GetIRKey(Code, KeyNr)) return true;
			return false;
		}

		// --- ActDevice : Auslesen der DeviceDaten ----------------------

		public DeviceData ActDevice {
			get {
				DeviceData dd;
				dd.Type     = (int)rbGetActDeviceType(iHandle);
				dd.SerialNr = (int)rbGetActDeviceSerialNr(iHandle);
				uint f  = rbGetActDeviceFirmwareNr(iHandle);
				dd.Firmware = String.Format("{0}.{1:00}.{2:00}.{3:00}", 
					(f & 0xFF000000) >> 24, (f & 0xFF0000) >> 16, 
					(f & 0xFF00) >> 8,      (f & 0xFF));
				dd.Name = Convert.ToString((IFTypen)dd.Type);
				return dd;
			}
		}

		// --- GetAnalog : Auslesen Analogwert -------------------------------

		internal int GetAnalog(Inp AnalogNr) {return GetAnalog((int)AnalogNr);}
		internal int GetAnalog(int AnalogNr) {
			if(FDoEvents) Application.DoEvents();
			if(!FOpen) throw(new FishFaceException("KeinOpen.GetAnalog"));
			int res = rbGetAnalog(iHandle, AnalogNr);
			if((uint)res == rbFehler) 
				throw(new FishFaceException("InterfaceProblem.GetAnalog"));
			return res;
		}
		internal int GetAnalogPur(int AnalogNr) {
			return GetAnalog(AnalogNr);
		}

		// --- GetCounter : Auslesen eines ImpulsCounts ---------------------

		internal int GetCounter(Inp InputNr) {return GetCounter((int)InputNr);}
		internal int GetCounter(int InputNr) {
			if(!FOpen) throw(new FishFaceException("KeinOpen.GetCounter"));
			int res = rbGetCounter(iHandle, InputNr);
			if((uint)res == rbFehler)
				throw(new FishFaceException("InterfaceProblem.ClearCounter"));
			return res;
		}

		// --- GetInput : Abfragen des Status eines I-Einganges --------------

		internal bool GetInput(Inp InputNr) {return GetInput((int)InputNr);}
		internal bool GetInput(int InputNr) {
			if(FDoEvents) Application.DoEvents();
			if(!FOpen) throw(new FishFaceException("KeinOpen.GetInput"));
			int res = rbGetInput(iHandle, InputNr);
			if((uint)res == rbFehler)
				throw(new FishFaceException("InterfaceProblem.GetInput"));
			return (res == 1) ? true : false;
		}

		// --- GetInputs : Auslesen des Status aller I-Eingnge ---------------

		internal int GetInputs() {
			if(!FOpen) throw(new FishFaceException("KeinOpen.GetInputs"));
			if(FDoEvents) Application.DoEvents();
			int res = rbGetInputs(iHandle);
			if((uint)res == rbFehler)
				throw(new FishFaceException("InterfaceProblem.GetInputs"));
			return res;
		}

		// --- GetIRKey : Auslesen Wert der IR Control --------------------------

		internal bool GetIRKey(IRCode Code, IRKeys KeyNr) {
			return GetIRKey((int)Code, (int)KeyNr);}
		internal bool GetIRKey(int Code, int KeyNr) {
			if(FDoEvents) Application.DoEvents();
			if(!FOpen) throw(new FishFaceException("KeinOpen.GetIRKey"));
			int res = rbGetIRKey(iHandle, Code, KeyNr);
			if((uint)res == rbFehler) 
				throw(new FishFaceException("InterfaceProblem.GetIRKey"));
			return (res == 1) ? true : false;
		}

		// --- GetVoltage : Auslesen Spannungswert -------------------------------

		internal int GetVoltage(Inp VoltNr) {return GetVoltage((int)VoltNr);}
		internal int GetVoltage(int VoltNr) {
			if(FDoEvents) Application.DoEvents();
			if(!FOpen) throw(new FishFaceException("KeinOpen.GetVoltage"));
			int res = rbGetVoltage(iHandle, VoltNr);
			if((uint)res == rbFehler) 
				throw(new FishFaceException("InterfaceProblem.GetVoltage"));
			return res;
		}

		// --- NotHalt : Abbruchwunsch -----------------------------------------

		public bool NotHalt {
			get{return FNotHalt;}
			set{FNotHalt = value;}
		}

		// --- OpenInterface : Verbindung zum Interface herstellen ------------

		internal void OpenInterface(IFTypen ifTyp, int SerialNr) {
			OpenInterface(ifTyp, SerialNr, true);
		}
		internal void OpenInterface(IFTypen ifTyp, int SerialNr, bool DoEvents) {
			iHandle = rbOpenInterfaceUSB((int)ifTyp, SerialNr);
			if((uint)iHandle == rbFehler)
				throw(new FishFaceException("InterfaceProblem.Open"));
			rbSetMotors(iHandle, 0);
			FNotHalt = false;
			FOpen = true;
			FDoEvents = DoEvents;
		}
		internal void OpenInterface(IFTypen ifTyp, Port ComNr, int AnalogZyklen) {
			OpenInterface(ifTyp, ComNr, AnalogZyklen, true);
		}
		internal void OpenInterface(IFTypen ifTyp,
			Port ComNr, int AnalogZyklen, bool DoEvents) {
			OpenInterface(ifTyp, (int)ComNr, AnalogZyklen, DoEvents);}
		internal void OpenInterface(IFTypen ifTyp, int ComNr, int AnalogZyklen) {
			OpenInterface(ifTyp, ComNr, AnalogZyklen, true);
		}
		internal void OpenInterface(IFTypen ifTyp, 
			int ComNr, int AnalogZyklen, bool DoEvents) {
			iHandle = rbOpenInterfaceCOM((int)ifTyp, ComNr, AnalogZyklen);
			if((uint)iHandle == rbFehler)
				throw(new FishFaceException("InterfaceProblem.Open"));
			rbSetMotors(iHandle, 0);
			FNotHalt = false;
			FOpen = true;
			FDoEvents = DoEvents;
		}

		// --- Outputs : Auslesen aller M-Ausgnge ---------------------------

		internal int Outputs {
			get {
				if (!FOpen) throw(new FishFaceException("KeinOpen.Outputs"));
				int res = rbGetMotors(iHandle);
				if ((uint)res == rbFehler)
					throw(new FishFaceException("InterfaceProblem.Outputs"));
				return res;
			}
			set {
				if (!FOpen) throw(new FishFaceException("KeinOpen.Outputs"));
				uint res = rbSetMotors(iHandle, value);
				if (res == rbFehler)
					throw(new FishFaceException("InterfaceProblem.Outputs"));
			}
		}

		// --- Pause : Programm anhalten --------------------------------------
	
		public void Pause(int mSek) {
			if (!FOpen) throw(new FishFaceException("KeinOpen.Pause"));
			uint sZeit = GetTickCount();
			do {
				if(FDoEvents) Application.DoEvents();
				if ((uint)GetInputs() == rbFehler)
					throw(new FishFaceException("InterfaceProblem.Pause"));
				if (FNotHalt || GetAsyncKeyState(ESCAPE) !=0)return;
				Sleep(rbPollInterval);
			}	while (GetTickCount() - sZeit < mSek);
		}
		
		// --- SetCounter : Setzen ImpulsZhler ------------------------------

		internal void SetCounter(Inp InputNr, int ICount) {
			SetCounter((int)InputNr, ICount);}
		internal void SetCounter(int InputNr, int ICount) {
			if(!FOpen) throw(new FishFaceException("KeinOpen.SetCounter"));
			if(rbSetCounter(iHandle, InputNr, ICount) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.SetCounter"));
		}

		// --- SetLamp : Setzen eines O-Ausganges.

		internal void SetLamp(Out LampNr, Dir OnOff) {
			SetLamp((int)LampNr, (int)OnOff);}
		internal void SetLamp(int LampNr, int OnOff) {
			SetLamp(LampNr, OnOff, 7);}
		internal void SetLamp(Out LampNr, Dir OnOff, int Power) {
			SetLamp((int)LampNr, (int)OnOff, Power);}
		internal void SetLamp(int LampNr, int OnOff, int Power) {
			if(FDoEvents) Application.DoEvents();
			if (!FOpen) throw(new FishFaceException("KeinOpen.SetLamp"));
			if(rbSetLampEx(iHandle, LampNr, OnOff, Power) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.SetLamp"));
		}

		// --- SetMotor : Setzen eines M-Ausganges ---------------------------

		internal void SetMotor(Out MotorNr, Dir Direction, Speed Speed, int ICount) {
			SetMotor((int)MotorNr, (int)Direction, (int)Speed, ICount);}
		internal void SetMotor(int MotorNr, int Direction, int Speed, int ICount) {
			if(FDoEvents) Application.DoEvents();
			if (!FOpen) throw(new FishFaceException("KeinOpen.SetMotor"));
			if(rbRobMotor(iHandle, MotorNr, Direction, Speed, ICount) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.SetMotor"));
		}
		internal void SetMotor(Out MotorNr, Dir Direction, Speed Speed) {
			SetMotor((int)MotorNr, (int)Direction, (int)Speed);}
		internal void SetMotor(int MotorNr, int Direction, int Speed) {
			if(FDoEvents) Application.DoEvents();
			if (!FOpen) throw(new FishFaceException("KeinOpen.SetMotor"));
			if(rbSetMotorEx(iHandle, MotorNr, Direction, Speed) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.SetMotor"));
		}
		internal void SetMotor(Out MotorNr, Dir Direction) {
			SetMotor((int)MotorNr, (int)Direction);}
		internal void SetMotor(int MotorNr, int Direction) {
			if(FDoEvents) Application.DoEvents();
			if (!FOpen) throw(new FishFaceException("KeinOpen.SetMotor"));
			if(rbSetMotor(iHandle, MotorNr, Direction) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.SetMotor"));
		}

		// --- SetMotors : Setzen aller M-Ausgnge ----------------------------

		internal void SetMotors(int MotorStatus, 
			int SpeedStatus, int SpeedStatus16, int ModeStatus) {
			if(FDoEvents) Application.DoEvents();
			if (!FOpen) throw(new FishFaceException("KeinOpen.SetMotors"));
			if(rbRobMotors(iHandle, MotorStatus, 
				(uint)SpeedStatus, SpeedStatus16, ModeStatus) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.SetMotors"));
		}
		internal void SetMotors(int MotorStatus, 
			int SpeedStatus, int SpeedStatus16) {
			if(FDoEvents) Application.DoEvents();
			if (!FOpen) throw(new FishFaceException("KeinOpen.SetMotors"));
			if(rbSetMotorsEx(iHandle, MotorStatus, 
				SpeedStatus, SpeedStatus16) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.SetMotors"));
		}
		internal void SetMotors(int MotorStatus) {
			if(FDoEvents) Application.DoEvents();
			if (!FOpen) throw(new FishFaceException("KeinOpen.SetMotors"));
			if(rbSetMotors(iHandle, MotorStatus) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.SetMotors"));
		}
		
		// --- Version --------------------------------------------------------------

		public static string Version {
			get{return "v4.1.99";}
		}

		// --- WaitForChange : Warte auf NrOfChanges Impulse von InputNr -----------

		internal void WaitForChange(Inp InputNr, int NrOfChanges) {
			WaitForChange((int)InputNr, NrOfChanges);}
		internal void WaitForChange(int InputNr, int NrOfChanges) {
			WaitForChange(InputNr, NrOfChanges, 0);
		}
		internal void WaitForChange(Inp InputNr, int NrOfChanges, Inp TermInputNr) {
			WaitForChange((int)InputNr, NrOfChanges, (int)TermInputNr);}
		internal void WaitForChange(int InputNr, int NrOfChanges, int TermInputNr) {
			if (!FOpen) throw(new FishFaceException("KeinOpen.WaitForChange"));
			rbSetCounter(iHandle, InputNr, 0);

			while (rbGetCounter(iHandle, InputNr) < NrOfChanges) {		
				if(FNotHalt || GetAsyncKeyState(ESCAPE) !=0) return;
				if(TermInputNr>0) if(GetInput(TermInputNr)) return;
				if ((uint)GetInputs() == rbFehler)
					throw(new FishFaceException("InterfaceProblem.WaitForChange"));
				if(FDoEvents) Application.DoEvents();
			}
		}

		// --- WaitForInput : Warten auf InputNr = OnOff ---------------------------

		internal void WaitForInput(Inp InputNr) {WaitForInput((int)InputNr);}
		internal void WaitForInput(int InputNr) {
			WaitForInput(InputNr, true);
		}
		internal void WaitForInput(Inp InputNr, bool OnOff) {
			WaitForInput((int)InputNr, OnOff);}
		internal void WaitForInput(int InputNr, bool OnOff) {
			if (!FOpen) throw(new FishFaceException("KeinOpen.WaitForInput"));
			do {
				if(FNotHalt || GetAsyncKeyState(ESCAPE) !=0) return;
				if ((uint)GetInputs() == rbFehler)
					throw(new FishFaceException("InterfaceProblem.WaitForInput"));
				Sleep(rbPollInterval);
				if(FDoEvents) Application.DoEvents();
			}   while(!GetInput(InputNr) == OnOff);
		}
		internal void WaitForInput(IRCode Code, IRKeys KeyNr) {
			WaitForInput((int)Code, (int)KeyNr);}
		internal void WaitForInput(int Code, int KeyNr) {
			WaitForInput(Code, KeyNr, true);
		}
		internal void WaitForInput(IRCode Code, IRKeys KeyNr, bool OnOff) {
			WaitForInput((int)Code, (int)KeyNr, OnOff);}
		internal void WaitForInput(int Code, int KeyNr, bool OnOff) {
			if (!FOpen) throw(new FishFaceException("KeinOpen.WaitForInput"));
			do {
				if(FNotHalt || GetAsyncKeyState(ESCAPE) !=0) return;
				if ((uint)GetInputs() == rbFehler)
					throw(new FishFaceException("InterfaceProblem.WaitForInput"));
				Sleep(rbPollInterval);
				if(FDoEvents) Application.DoEvents();
			}   while(!GetIRKey(Code, KeyNr) == OnOff);
		}

		// --- WaitForMotor : Warten auf MotorReady-Ereignis oder Ablauf mSek -------

		internal Wait WaitForMotor(int mSek, int MotorNr) {
			uint t;

			if(!FOpen) throw(new FishFaceException("KeinOpen.WaitForMotor"));
			t = GetTickCount() + Convert.ToUInt32(mSek);
			do {
				if(rbGetModeStatus(iHandle, MotorNr) == 0) return Wait.Ende;
				if(FNotHalt || (GetAsyncKeyState(ESCAPE) != 0))
					rbSetMotor(iHandle, MotorNr, 0);
				if(FNotHalt) return Wait.NotHalt;
				if(GetAsyncKeyState(ESCAPE) != 0)return Wait.ESC;
				if(FDoEvents) Application.DoEvents();
				Sleep(Convert.ToUInt32(rbPollInterval));
				if ((uint)GetInputs() == rbFehler)
					throw(new FishFaceException("InterfaceProblem.WaitForMotor"));
			}   while((mSek == 0) || (t>GetTickCount()));
			return Wait.Time;
		}
	}
}
