// ------------------------------------------------------------------------------------
// FishDev40.CS                                                 Ulrich Mller 16.06.05
//                                                              PCS1700   C# Final SP2
// Assembly FishDevices40 mit Funktionen zum Betrieb der fischertechnik Interfaces
// Die Resourcen eines Interfaces sind Objekte, die ereignisgesteuert genutzt werden.
// Klasse FishDevices       : Verwaltung der Resourcen eines Interfaces
//                            abgeleitet von FishBase mit Zugriffsfunktionen auf
//                            das Interface
// Klasse FishDevException  : throw FishDevices Ausnahmen.
// Klasse DeviceBase        : Basisklasse fr die Devices
// -----------------------------------------------------------------------------------

using System;
using System.Collections;
using System.Threading;

namespace FishDevices40 {

	// FishDevExceptions : Exceptions, die durch FishDevices und die Device Klassen 
	// ausgelst werden
	public class FishDevException: ApplicationException {
		public FishDevException(){}
		public FishDevException(string message) : base(message){}
		public FishDevException(string message, Exception inner) : base(message, inner){}
	}
	
	// DeviceBase : Basisklasse fr die Devices 
 	public class DeviceBase {
		protected FishDevices fd;
		internal  Thread      deviceThread;
		internal  int         threadInterval;
		private   int         deviceNumber;
		private   bool        withEvents;

		public DeviceBase(FishDevices fd, bool WithEvents, int DeviceNumber) {
			this.fd           = fd;
			this.deviceNumber = DeviceNumber;
			this.withEvents   = WithEvents;
			this.deviceThread = null;
		}
		public int DeviceNumber {
			get{return deviceNumber;}
		}
		public bool WithEvents {
			get{return withEvents;}
			set{withEvents = value;}
		}
		public int ThreadInterval {
			get{return threadInterval;}
			set{threadInterval = value < 11 ? 11 : value;}
		}
	}

	// FishDevices ------------------------------------------------
	//  Verwaltung der Resourcen eines fischertechnik Interfaces.
	//  Connect / DisConnect, Start / Suspend / Resume.
	//  Fhren einer Liste der aktiven Devices.
	//  Bereitstellen von Zugriffsfunktionen (Modifier internal)
	// -----------------------------------------------------------
	public class FishDevices : FishBase {
		private ArrayList DeviceList = new ArrayList(32);

		// FishDevices : leerer Constructor
		public FishDevices() : base() {}
		
		internal void AddDevice(DeviceBase DeviceObject) {
			DeviceList.Add(DeviceObject);
		}

		public void Connect(IFTypen ifTyp, int SerialNr, bool DoEvents) {		
			try {
				base.OpenInterface(ifTyp, SerialNr, DoEvents);
			}
			catch(FishFaceException) {
				throw(new FishDevException("InterfaceProblem.Connect"));
			}
		}
		public void Connect(IFTypen ifTyp, int SerialNr) {
			this.Connect(ifTyp, SerialNr, true);
		}
		public void Connect(IFTypen ifTyp, Port ComNr, int AnalogZyklen, bool DoEvents) {
			this.Connect(ifTyp, (int)ComNr, AnalogZyklen, DoEvents);
		}
		public void Connect(IFTypen ifTyp, int ComNr, int AnalogZyklen, bool DoEvents) {
			try {
				base.OpenInterface(ifTyp, ComNr, AnalogZyklen, DoEvents);
			}
			catch(FishFaceException) {
				throw(new FishFaceException("InterfaceProblem.Connect"));
			}
		}
		public void Connect(IFTypen ifTyp, Port ComNr, int AnalogZyklen) {
			this.Connect(ifTyp, ComNr, AnalogZyklen, true);
		}
		public void Connect(IFTypen ifTyp, int ComNr, int AnalogZyklen) {
			this.Connect(ifTyp, ComNr, AnalogZyklen, true);
		}
		
		// DisConnect : Beenden der DeviceThreads durch NotHalt = true
		//              Beenden der Interface-Verbindung.
		public void DisConnect() {
			base.NotHalt = true;
			base.Pause(555);
			base.Outputs = 0;
			base.CloseInterface();
		}

		// Start : Starten der DeviceThreads --------------------------------
		public void Start() {
			foreach (DeviceBase db in DeviceList)
				if(db.WithEvents) db.deviceThread.Start();
		}

		// Resume : Wiederaufnahme der DeviceThreads ------------------------
		public void Resume() {
			foreach(DeviceBase db in DeviceList)
				if(db.WithEvents) db.deviceThread.Resume();
		}

		// Suspend : Parken der DeviceThreads -------------------------------
		public void Suspend() {
			foreach(DeviceBase db in DeviceList)
				if(db.WithEvents) db.deviceThread.Suspend();
			base.Outputs = 0;
		}
	}
}
