// --------------------------------------------------------------------------------------
//
// IRInput.CS                                                      Ulrich Mller 21.06.05
//                                                                 PCS1700   C# Final SP2
// Eingnge des IR-Empfngers
//
// --------------------------------------------------------------------------------------

using System;
using System.Threading;

namespace FishDevices40 {
	public class IRInput : DeviceBase {

		public delegate void Changed(object sender);
		public event         Changed ChangedToTrue;
		public event         Changed ChangedToFalse;

		private bool actValue, oldValue;
		private int  Kode;

		// --- Constructor ------------------------------------------------------

		public IRInput(FishDevices fd, IRCode Kode, IRKeys KeyNr)
			: this(fd, true, Convert.ToInt32(Kode), Convert.ToInt32(KeyNr)) {}
		public IRInput(FishDevices fd, int Kode, int KeyNr) 
			: this(fd, true, Kode, KeyNr) {}
		public IRInput(FishDevices fd, bool WithEvents, IRCode Kode, IRKeys KeyNr) 
			: this(fd, WithEvents, Convert.ToInt32(Kode), Convert.ToInt32(KeyNr)) {}
		public IRInput(FishDevices fd, bool WithEvents, int Kode, int KeyNr) 
			: base(fd, WithEvents, KeyNr) {
			this.Kode = Kode;
			base.threadInterval = 22;
			fd.AddDevice(this);
			if(base.WithEvents) {
				base.deviceThread = new Thread(new ThreadStart(EventLoop));
				base.deviceThread.IsBackground = true;
			}
		}

		// --- EventLoop : Scannen des IR-Einganges und Auslsen von Ereignissen ---

		private void EventLoop() {
			oldValue = fd.GetIRKey(this.Kode, base.DeviceNumber);
			while(!fd.NotHalt && !fd.ESC) {
				if((uint)fd.GetInputs() == FishBase.rbFehler) 
					throw(new FishDevException("InterfaceProblem.IRInput"));
				actValue = fd.GetIRKey(this.Kode, base.DeviceNumber);
				if(actValue && oldValue == false) 
					if(ChangedToTrue  != null) ChangedToTrue(this);
				if(!actValue && oldValue == true) 
					if(ChangedToFalse != null) ChangedToFalse(this);
				oldValue = actValue;
				Thread.Sleep(base.threadInterval);
			}
		}

		// --- Eigenschaften des IR-Einganges ----------------------

		public int IRKeyNr {
			get{return base.DeviceNumber;}
		}
		public int IRCodeNr {
			get{return this.Kode;}
		}

		public bool IsTrue {
			get{
				try{return fd.GetIRKey(this.Kode, base.DeviceNumber);}
				catch(FishFaceException eft) {
					throw(new FishDevException(eft.Message + ".IRInput")); }
			}
		}

		// --- Methoden zum Warten auf Vernderungen des IR-Einganges ------------
		
		public void WaitForKey() {this.WaitForKey(true);}
		public void WaitForKey(bool OnOff) { 
			try{fd.WaitForInput(this.Kode, base.DeviceNumber, OnOff);}
			catch(FishFaceException eft) {
				throw(new FishDevException(eft.Message + ".BinaryInput.WaitForInput")); }
		}
	}
}
