// --------------------------------------------------------------------------------------
//
// ImpulseSensor.CS                                                Ulrich Mller 16.06.05
//                                                                 PCS1700   C# Final SP2
// Zhlen von Impulsen an einem I-Eingang
//
// --------------------------------------------------------------------------------------

using System;
using System.Threading;

namespace FishDevices40 {
	public class ImpulseSensor : BinaryInput {

		private int startCount;
		private int destCount;
		private int actCount;
		private int nrOfChanges;
		private int termInputNr;
		private int stateValue;

		public delegate void ImpulseCount(object sender, int ActualCount, bool Ready);
		public event         ImpulseCount ChangedCount;
		public event         ImpulseCount FinalCount;

		// --- Constructor ----------------------------------------------
		
		public ImpulseSensor(FishDevices fd, Inp InputNumber) 
			: this(fd, true, Convert.ToInt32(InputNumber)) {}
		public ImpulseSensor(FishDevices fd, int InputNumber) 
			: this(fd, true, InputNumber) {}
		public ImpulseSensor(FishDevices fd, bool WithEvents, Inp InputNumber) 
			: this(fd, WithEvents, Convert.ToInt32(InputNumber)) {}
		public ImpulseSensor(FishDevices fd, bool WithEvents, int InputNumber) 
			: base(fd, WithEvents, InputNumber) {
			this.startCount  = 0;
			this.termInputNr = 0;
			this.stateValue  = 0;
			base.threadInterval = 11;
			if(base.WithEvents) {
				base.deviceThread = new Thread(new ThreadStart(EventLoop));
				base.deviceThread.IsBackground = true;
			}
		}
		public ImpulseSensor(FishDevices fd, Inp InputNumber, BinaryInput TermSensor) 
			: this(fd, true, Convert.ToInt32(InputNumber), TermSensor) {}
		public ImpulseSensor(FishDevices fd, int InputNumber, BinaryInput TermSensor) 
			: this(fd, true, InputNumber, TermSensor) {}
		public ImpulseSensor(FishDevices fd, bool WithEvents, 
			Inp InputNumber, BinaryInput TermSensor) 
			: this(fd, WithEvents, Convert.ToInt32(InputNumber), TermSensor) {}
		public ImpulseSensor(FishDevices fd, bool WithEvents, 
			int InputNumber, BinaryInput TermSensor) 
			: base(fd, WithEvents, InputNumber) {
			this.startCount  = 0;
			this.termInputNr = TermSensor.InputNumber;
			this.stateValue  = 0;
			base.threadInterval = 11;
			if(base.WithEvents) {
				base.deviceThread = new Thread(new ThreadStart(EventLoop));
				base.deviceThread.IsBackground = true;
			}
		}

		// --- Ereignis-Bearbeitung ----------------

		private void EventLoop() {
			bool Ready;
			while(!fd.NotHalt && !fd.ESC) {
				Ready = false;
				if((uint)fd.GetInputs() == FishBase.rbFehler) 
					throw(new FishDevException("InterfaceProblem.ImpulseSensor"));
				if(startCount < destCount) {
					actCount = startCount + fd.GetCounter(base.InputNumber);
					if(actCount >= destCount) {
						Ready = true;
						startCount = destCount;
					}
				}
				else {
					actCount = startCount - fd.GetCounter(base.InputNumber);
					if(actCount <= destCount) {
						Ready = true;
						startCount = actCount;
					}
				}
				if(Ready) stateValue = 0;
				if(ChangedCount != null) ChangedCount(this, actCount, Ready);
				if(FinalCount != null && Ready) FinalCount(this, actCount, true);
				Thread.Sleep(base.threadInterval);
			}
		}

		// --- Eigenschaften -------------------------------

		public int CurrentCounter {
			get{return startCount;}
			set{startCount = value;}
		}

		// --- Methoden -----------------------------------------

		public void CountChanges(int NrOfChanges) {
			this.nrOfChanges = NrOfChanges;
			this.startCount  = 0;
			this.destCount   = NrOfChanges;
			this.stateValue  = 1;
			fd.SetCounter(base.InputNumber, 0);
		}
		public void CountPosition(int DestCount) {
			this.nrOfChanges = Math.Abs(DestCount-startCount);
			this.destCount   = DestCount;
			this.stateValue  = 2;
			fd.SetCounter(base.InputNumber, 0);
		}
		public void WaitForDone() {
			try{
				fd.WaitForChange(base.InputNumber, nrOfChanges, termInputNr);
				this.startCount = destCount;
				this.stateValue = 0;
			}
			catch(FishFaceException eft) {
				throw(new FishDevException(eft.Message + ".ImpulseSensor.WaitForDone"));
			}
		}
	}
}
