// --------------------------------------------------------------------------------------
//
// LightBarrier.CS                                                 Ulrich Mller 16.06.05
//                                                                 PCS1700   C# Final SP2
// LightBarrier : Lichtschranke aus Lampe und Photowiderstand
//
// --------------------------------------------------------------------------------------

using System;
using System.Threading;

namespace FishDevices40 {
	public class LightBarrier	: DeviceBase {
		private DLamp           BDLamp;
		private Lamp            BLamp;
		private PhotoTransistor Photo;

		public delegate void Changed(object sender);
		public event         Changed ChangedToFree;
		public event         Changed ChangedToBroken;

		// --- Constructor -----------------------------------------------

		public LightBarrier(FishDevices fd, Lamp BLamp, PhotoTransistor Photo) 
			: this(fd, false, BLamp, Photo) {}
		public LightBarrier(FishDevices fd, bool WithEvents, 
			Lamp BLamp, PhotoTransistor Photo)
			: base(fd, false, 101) {
			fd.AddDevice(this);
			this.BDLamp = null;
			this.BLamp  = BLamp;
			this.Photo  = Photo;
			Photo.ChangedToFalse += new BinaryInput.Changed(BrokenEvent);
			Photo.ChangedToTrue  += new BinaryInput.Changed(FreeEvent);
		}
		public LightBarrier(FishDevices fd, DLamp BDLamp, PhotoTransistor Photo) 
			: this(fd, true, BDLamp, Photo) {}
		public LightBarrier(FishDevices fd, bool WithEvents, 
			DLamp BDLamp,	PhotoTransistor Photo) 
			: base(fd, false, 101)	{
			fd.AddDevice(this);
			this.BDLamp = BDLamp;
			this.BLamp  = null;
			this.Photo  = Photo;
			if(WithEvents) {
				Photo.ChangedToFalse += new BinaryInput.Changed(BrokenEvent);
				Photo.ChangedToTrue  += new BinaryInput.Changed(FreeEvent);
			}
		}

		// --- Ereignisroutinen -----------------------------------------

		private void FreeEvent(object sender) {
			if(ChangedToFree != null) ChangedToFree(this);
		}
		private void BrokenEvent(object sender) {
			if(ChangedToBroken != null) ChangedToBroken(this);
		}

		// --- Eigenschaften --------------------------------------------

		public int PhotoNumber {
			get{return Photo.InputNumber;}
		}
		public int LampNumber {
			get{return BLamp == null ? BDLamp.OutputNumber : BLamp.OutputNumber;}
		}
		public bool IsFree {
			get{return Photo.IsTrue;}
		}
		public bool IsBroken {
			get{return !Photo.IsTrue;}
		}

		// --- Methoden --------------------------------------------------

		public void On() {
			if(BDLamp != null) BDLamp.On(); else BLamp.On();
		}
		public void Off() {
			if(BDLamp != null) BDLamp.Off(); else BLamp.Off();
		}
		public void WaitForBroken() {
			Photo.WaitForInput(false);
		}
		public void WaitForFree() {
			Photo.WaitForInput(true);
		}
		public void WaitForPassed() {
			this.WaitForBroken();
			this.WaitForFree();
		}
	}
}
