// --------------------------------------------------------------------------------------
//
// LimitedMotor.CS                                                 Ulrich Mller 16.06.05
//                                                                 PCS1700   C# Final SP2
// LimitedMotor : Motor dessen Fahrweg in beiden Richtungen durch 
//                BinaryInputs begrenzt wird
//
// --------------------------------------------------------------------------------------

using System;

namespace FishDevices40 {
	public class LimitedMotor	: DeviceBase {

		private Motor       LMotor;
		private BinaryInput leftSensor;
		private BinaryInput rightSensor;
		private Dir         stateValue;

		public delegate void Changed(object sender);
		public event         Changed LeftToTrue;
		public event         Changed RightToTrue;

		// --- Constructor -----------------------------------------------

		public LimitedMotor(FishDevices fd, Motor Mot,
			BinaryInput ELeft, BinaryInput ERight) 
			: this(fd, true, Mot, ELeft, ERight) {}
		public LimitedMotor(FishDevices fd, bool WithEvents, Motor Mot, 
			BinaryInput ELeft, BinaryInput ERight) : base(fd, false, 102) {
			fd.AddDevice(this);
			this.LMotor      = Mot;
			this.leftSensor  = ELeft;
			this.rightSensor = ERight;
			this.stateValue  = Dir.Off;
			if(WithEvents) {
				rightSensor.ChangedToTrue += new BinaryInput.Changed(RightEvent);
				leftSensor.ChangedToTrue  += new BinaryInput.Changed(LeftEvent);
			}
		}

		// --- Ereignis-Routinen -----------------------------

		private void RightEvent(object sender) {
			LMotor.Off();
			stateValue = Dir.Left;
			if(LeftToTrue != null) LeftToTrue(leftSensor);
		}
		private void LeftEvent(object sender) {
			LMotor.Off();
			stateValue = Dir.Right;
			if(RightToTrue != null) RightToTrue(rightSensor);
		}

		// --- Eigenschaften ------------------

		public int MotorNumber {
			get{return LMotor.OutputNumber;}
		}
		public int LeftSwitch {
			get{return leftSensor.InputNumber;}
		}
		public int RightSwitch {
			get{return rightSensor.InputNumber;}
		}
		public int MotorState {
			get{return LMotor.State;}
		}
		public bool IsLeft {
			get {return (stateValue == Dir.Left);}
		}
		public bool IsRight {
			get{return (stateValue == Dir.Right);}
		}

		// --- Methoden --------------------------------------

		public void GoLeft(Speed speed) {
			stateValue = Dir.Off;
			LMotor.Go(Dir.Left, speed);
		}
		public void GoLeft(int Speed) {
			stateValue = Dir.Off;
			LMotor.Go(1, Speed);
		}
		public void GoLeft() {
			stateValue = Dir.Off;
			LMotor.Left();
		}

		public void GoRight(Speed speed) {
			stateValue = Dir.Off;
			LMotor.Go(Dir.Right, speed);
		}
		public void GoRight(int Speed) {
			stateValue = Dir.Off;
			LMotor.Go(2, Speed);
		}
		public void GoRight() {
			stateValue = Dir.Off;
			LMotor.Right();
		}

		public void WaitForDone() {
			if(LMotor.State == 1) {
				leftSensor.WaitForInput();
				stateValue = Dir.Left;
			}
			else {
				rightSensor.WaitForInput();
				stateValue = Dir.Right;
			}
			LMotor.Off();
		}
	}
}
