// -------------------------------------------------------------------------------------
//
// MR2Event.CS                                                    Ulrich Mller 18.06.05
//                                                                PCS1700   C# Final SP2
// MR2 : Mobile Robot vom Trusty Typ
//       Steuerung und Antrieb durch zwei Radmotoren mit Impulszhlern
//       und links und rechts je einen Sensor (Taster)
//       Zustzlich zwei Eingnge fr den IR-Sender.
//       Betrieb ber den ROBO RF Datalink. Mit dem IR-Sender knnen festgefahrene
//       Positionen aufgelst werden.
//       Die Versorgungsspannung wird laufend angezeigt, das Unterschreiten eines
//       Grenzwertes wird signalisiert, es wird aber nicht eingegriffen.
//
// Programmierung erfolgte ber Events
// Einsatz : FishDevices40.DLL / umFish40.DLL
//
// -------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using FishDevices40;

namespace MR2Event {
	public class frmMain : System.Windows.Forms.Form {
		private System.Windows.Forms.Label lblStatus;
		private System.Windows.Forms.Button cmdAction;
		private System.ComponentModel.Container components = null;

		// -------------- RoboFace : Klassendeklarationen ------------------------
		
		private FishDevices   fd;
		private Motor         leftMotor;
		private Motor         rightMotor;
		private ImpulseSensor impulseSensor;
		private Sensor        leftSensor;
		private Sensor        rightSensor;
		private IRInput       backLeft;
		private IRInput       backRight;
		private VoltageInput  powerSupply;
		private bool          BusyInTurn;

		private const int BackCount = 8;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label lblPower;
		private System.Windows.Forms.Label lblPowerFail;
		private System.Windows.Forms.Label lblActDevice;
		private const int TurnCount = 8;

		public frmMain() {
			InitializeComponent();
			// --- RoboFace : Instanzierung, Ereignisse -----------------------------
			fd            = new FishDevices();
			leftMotor     = new Motor(fd, Out.M1);
			rightMotor    = new Motor(fd, Out.M2);
			impulseSensor = new ImpulseSensor(fd, 1);
			leftSensor    = new Sensor(fd, Inp.I4);
			rightSensor   = new Sensor(fd, Inp.I3);
			backLeft      = new IRInput(fd, IRCode.Code1, IRKeys.M1BW);
			backRight     = new IRInput(fd, IRCode.Code1, IRKeys.M1FW);
			powerSupply   = new VoltageInput(fd, true, Inp.AV, 2.50, 4.00);
			BusyInTurn    = false;
			
			leftSensor.ChangedToTrue    += new BinaryInput.Changed(TurnLeft);
			rightSensor.ChangedToTrue   += new BinaryInput.Changed(TurnRight);
			backLeft.ChangedToTrue      += new IRInput.Changed(BackLeft);
			backRight.ChangedToTrue     += new IRInput.Changed(BackRight);
			powerSupply.ChangedToLow    += new VoltageInput.Limit(PowerFail);
			powerSupply.ChangedToNormal += new VoltageInput.Limit(PowerNormal);
		}
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent()
		{
			this.lblStatus = new System.Windows.Forms.Label();
			this.cmdAction = new System.Windows.Forms.Button();
			this.label1 = new System.Windows.Forms.Label();
			this.lblPower = new System.Windows.Forms.Label();
			this.lblPowerFail = new System.Windows.Forms.Label();
			this.lblActDevice = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// lblStatus
			// 
			this.lblStatus.Font = new System.Drawing.Font("Tahoma", 11.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lblStatus.Location = new System.Drawing.Point(16, 16);
			this.lblStatus.Name = "lblStatus";
			this.lblStatus.Size = new System.Drawing.Size(344, 32);
			this.lblStatus.TabIndex = 0;
			this.lblStatus.Text = "START drcken";
			this.lblStatus.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// cmdAction
			// 
			this.cmdAction.Location = new System.Drawing.Point(16, 128);
			this.cmdAction.Name = "cmdAction";
			this.cmdAction.Size = new System.Drawing.Size(344, 40);
			this.cmdAction.TabIndex = 1;
			this.cmdAction.Text = "&START";
			this.cmdAction.Click += new System.EventHandler(this.cmdAction_Click);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(24, 64);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(104, 24);
			this.label1.TabIndex = 2;
			this.label1.Text = "PowerSupply : ";
			// 
			// lblPower
			// 
			this.lblPower.BackColor = System.Drawing.Color.LightGray;
			this.lblPower.Location = new System.Drawing.Point(136, 64);
			this.lblPower.Name = "lblPower";
			this.lblPower.Size = new System.Drawing.Size(56, 16);
			this.lblPower.TabIndex = 3;
			this.lblPower.Text = "0";
			this.lblPower.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// lblPowerFail
			// 
			this.lblPowerFail.Location = new System.Drawing.Point(208, 64);
			this.lblPowerFail.Name = "lblPowerFail";
			this.lblPowerFail.Size = new System.Drawing.Size(16, 16);
			this.lblPowerFail.TabIndex = 4;
			// 
			// lblActDevice
			// 
			this.lblActDevice.Location = new System.Drawing.Point(24, 96);
			this.lblActDevice.Name = "lblActDevice";
			this.lblActDevice.Size = new System.Drawing.Size(328, 16);
			this.lblActDevice.TabIndex = 5;
			// 
			// frmMain
			// 
			this.AcceptButton = this.cmdAction;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.ClientSize = new System.Drawing.Size(376, 183);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.lblActDevice,
																		  this.lblPowerFail,
																		  this.lblPower,
																		  this.label1,
																		  this.cmdAction,
																		  this.lblStatus});
			this.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.MaximizeBox = false;
			this.Name = "frmMain";
			this.Text = "RoboFace : MR2-Event";
			this.ResumeLayout(false);

		}
		#endregion

		[STAThread]
		static void Main() 
		{
			Application.Run(new frmMain());
		}


		// --- MR2 : Betriebsprogramme ---------------------------

		private void RunBack(int Inc) {
			lblStatus.Text = "RunBack";
			leftMotor.Backward();
			rightMotor.Backward();
			impulseSensor.CountChanges(Inc);
			impulseSensor.WaitForDone();
		}
		private void LeftTurn(int Inc) {
			lblStatus.Text = "LeftTurn";
			leftMotor.Forward();
			rightMotor.Backward();
			impulseSensor.CountChanges(Inc);
			impulseSensor.WaitForDone();
		}
		private void RightTurn(int Inc) {
			lblStatus.Text = "RightTurn";
			leftMotor.Backward();
			rightMotor.Forward();
			impulseSensor.CountChanges(Inc);
			impulseSensor.WaitForDone();
		}
		private void RunForward() {
			lblStatus.Text = "RunForward";
			lblPower.Text  = powerSupply.ActualValue.ToString("0.00V");
			leftMotor.Forward();
			rightMotor.Forward();
		}

		// --- FishDevices40 : MR2 Eventroutinen -----------
		private void TurnLeft(object sender) {
			if (BusyInTurn) return;
			BusyInTurn = true;
			RunBack(BackCount);
			LeftTurn(TurnCount);
			RunForward();
			BusyInTurn = false;
		}
		private void TurnRight(object sender) {
			if (BusyInTurn) return;
			BusyInTurn = true;
			RunBack(BackCount);
			RightTurn(TurnCount);
			RunForward();
			BusyInTurn = false;
		}
		private void BackLeft(object sender) {
			RunBack(BackCount);
			LeftTurn(TurnCount);
			RunForward();
		}
		private void BackRight(object sender) {
			RunBack(BackCount);
			RightTurn(TurnCount);
			RunForward();
		}
		private void PowerFail(object sender, int actValue) {
			lblPowerFail.BackColor = Color.Red;
		}
		private void PowerNormal(object sender, int actValue) {
			lblPowerFail.BackColor = Color.GreenYellow;
		}
		
		// --- WinForm : MR2 Hauptprogramm ------------------------------

		private void cmdAction_Click(object sender, System.EventArgs e) {
			try {
				if(cmdAction.Text == "&START") {
					fd.Connect(IFTypen.ftROBO_first_USB, 0);
					DeviceData dd = fd.ActDevice;
					lblActDevice.Text = String.Format("{0} ({1} / {2} ) Firmware : {3}",
						dd.Name, dd.Type, dd.SerialNr, dd.Firmware);
					fd.Start();
					cmdAction.Text = "&HALT";
					lblStatus.Text = "--- MR2 gestartet ---";
					RunForward();
				}
				else {
					lblStatus.Text = "--- MR2 wird beendet ---";
					fd.DisConnect();
					this.Close();
				}
			}
			catch (FishDevException efd) {
				lblStatus.Text = efd.Message;
			}		
		}
	}
}
