// -------------------------------------------------------------------------------------
//
// MR2Linear.CS                                                   Ulrich Mller 18.06.05
//                                                                PCS1700   C# Final SP2
// MR2 : Mobile Robot vom Trusty Typ
//       Steuerung und Antrieb durch zwei Radmotoren (M1, M2) 
//       mit Impulszhlern (I1, I2) nur der am linken Rad wird ausgewertet
//       und links und rechts je einen Sensor (Taster I3, I4)
//
// Programmierung erfolgte linear
// Einsatz FishDevices40.DLL / umFish40.DLL
//
// -------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using FishDevices40;

namespace MR2Linear {
	public class frmMain : System.Windows.Forms.Form 	{
		private System.Windows.Forms.Label lblStatus;
		private System.Windows.Forms.Button cmdAction;
		private System.ComponentModel.Container components = null;

		// -------------- FishDevices : Klassendeklarationen ------------------------
		
		private FishDevices   fd;
		private Motor         leftMotor;
		private Motor         rightMotor;
		private ImpulseSensor impulseSensor;
		private Sensor        leftSensor;
		private Sensor        rightSensor;

		private const int BackCount = 8;
		private const int TurnCount = 8;

		public frmMain() {
			InitializeComponent();

			// --- FishDevices : Instanzierung, Ereignisse -----------------------------
			
			fd            = new FishDevices();
			leftMotor     = new Motor(fd, 1);
			rightMotor    = new Motor(fd, 2);
			impulseSensor = new ImpulseSensor(fd, false, 1);
			leftSensor    = new Sensor(fd, false, 4);
			rightSensor   = new Sensor(fd, false, 3);
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent()
		{
			this.lblStatus = new System.Windows.Forms.Label();
			this.cmdAction = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// lblStatus
			// 
			this.lblStatus.Font = new System.Drawing.Font("Tahoma", 11.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lblStatus.Location = new System.Drawing.Point(16, 8);
			this.lblStatus.Name = "lblStatus";
			this.lblStatus.Size = new System.Drawing.Size(336, 32);
			this.lblStatus.TabIndex = 0;
			this.lblStatus.Text = "START drcken";
			this.lblStatus.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// cmdAction
			// 
			this.cmdAction.Location = new System.Drawing.Point(16, 72);
			this.cmdAction.Name = "cmdAction";
			this.cmdAction.Size = new System.Drawing.Size(344, 40);
			this.cmdAction.TabIndex = 1;
			this.cmdAction.Text = "&START";
			this.cmdAction.Click += new System.EventHandler(this.cmdAction_Click);
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.ClientSize = new System.Drawing.Size(368, 133);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																																	this.cmdAction,
																																	this.lblStatus});
			this.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.MaximizeBox = false;
			this.Name = "Form1";
			this.Text = "RoboFace : MR2-Linear";
			this.ResumeLayout(false);

		}
		#endregion

		[STAThread]
		static void Main() 
		{
			Application.Run(new frmMain());
		}


		// --- MR2 : Betriebsprogramme ---------------------------

		private void RunBack(int Inc) {
			lblStatus.Text = "RunBack";
			leftMotor.Backward();
			rightMotor.Backward();
			impulseSensor.CountChanges(Inc);
			impulseSensor.WaitForDone();
		}
		private void LeftTurn(int Inc) {
			lblStatus.Text = "LeftTurn";
			leftMotor.Forward();
			rightMotor.Backward();
			impulseSensor.CountChanges(Inc);
			impulseSensor.WaitForDone();
		}
		private void RightTurn(int Inc) {
			lblStatus.Text = "RightTurn";
			leftMotor.Backward();
			rightMotor.Forward();
			impulseSensor.CountChanges(Inc);
			impulseSensor.WaitForDone();
		}
		private void RunForward() {
			lblStatus.Text = "RunForward";
			leftMotor.Forward();
			rightMotor.Forward();
		}
		
		// --- WinForm : MR2 Hauptprogramm ------------------------------

		private void cmdAction_Click(object sender, System.EventArgs e) {
			try {
				fd.Connect(IFTypen.ftROBO_first_USB, 0);
				cmdAction.Text    = "Ende : ESC";
				cmdAction.Enabled = false;
				lblStatus.Text = "--- MR2 gestartet, Ende : ESC ---";

				do {
					RunForward();
					if(leftSensor.IsTrue) {
						RunBack(BackCount);
						LeftTurn(TurnCount);
					}
					else if(rightSensor.IsTrue) {
						RunBack(BackCount);
						RightTurn(TurnCount);
					}
				} while(!fd.Finish());
				lblStatus.Text = "--- MR2 wird beendet ---";
				fd.DisConnect();
				this.Close();
			}
			catch (FishDevException efd) {
				lblStatus.Text = efd.Message;
			}
		}
	}
}
