// --------------------------------------------------------------------------------------
//
// MonoOutput.CS                                                   Ulrich Mller 16.06.05
//                                                                 PCS1700   C# Final SP2
// Devices mit Anschlu an nur einen Pol eines M-Ausganges
//
// --------------------------------------------------------------------------------------

using System;
using System.Threading;

namespace FishDevices40 {
	public class MonoOutput : DeviceBase {

		private int stateValue;

		// --- Constructor --------------------------------------

		public MonoOutput(FishDevices fd, Out OutputNumber) 
			: this(fd, Convert.ToInt32(OutputNumber)) {}
		public MonoOutput(FishDevices fd, int OutputNumber) 
			: base(fd, false, OutputNumber) {
			fd.AddDevice(this);
			this.stateValue = 0;
		}

		// --- Eigenschaften -----

		public int State {
			get{return stateValue;}
		}
		public int OutputNumber {
			get{return base.DeviceNumber;}
		}

		// --- Methoden ------------------------

		public void On() {
			stateValue = 1;
			try{
				fd.SetLamp(base.DeviceNumber, stateValue);
			}
			catch(FishFaceException eft) {
				throw(new FishDevException(eft.Message + ".MonoOutput.On"));
			}
		}
		public void Off() {
			stateValue = 0;
			try{
				fd.SetLamp(base.DeviceNumber, stateValue);
			}
			catch(FishFaceException eft) {
				throw(new FishDevException(eft.Message + ".MonoOutput.Off"));
			}
		}
	}

	// --- Lamp : Klasse fr eine Lampe ------------------------------------
	
	public class Lamp : MonoOutput {
		
		private int  freqOn, freqOff;
		private bool blinkingOn;
		private Thread BlinkingThread;

		public Lamp(FishDevices fd, Out OutputNumber) 
			: this(fd, Convert.ToInt32(OutputNumber)) {}
		public Lamp(FishDevices fd, int OutputNumber) : base(fd, OutputNumber) {}

		// --- Methoden -------------------

		public void BlinkingOn(int Freq) {
			this.BlinkingOn(Freq, Freq);
		}
		public void BlinkingOn(int FreqOn, int FreqOff) {
			this.freqOn  = FreqOn  > 11 ? FreqOn  : 11;
			this.freqOff = FreqOff > 11 ? FreqOff : 11;
			BlinkingThread = new Thread(new ThreadStart(BlinkingLoop));
			BlinkingThread.IsBackground = true;
			blinkingOn = true;
			BlinkingThread.Start();
		}
		public void BlinkingOff() {
			blinkingOn = false;
		}
		private void BlinkingLoop() {
			bool FlipFlop = true;
			int  BlinkFreq = freqOn;
			while(!fd.NotHalt && !fd.ESC && blinkingOn) {
				if(FlipFlop) {
					this.On();
					FlipFlop  = false;
					BlinkFreq = freqOn;
				}
				else {
					this.Off();
					FlipFlop  = true;
					BlinkFreq = freqOff;
				}
				Thread.Sleep(BlinkFreq);
			}
			this.Off();
		}
	}

	// --- Magnet : Klasse fr einen Magneten ------------------------------

	public class Magnet : MonoOutput {
		public Magnet(FishDevices fd, Out OutputNumber) 
			: this(fd, Convert.ToInt32(OutputNumber)) {}
		public Magnet(FishDevices fd, int OutputNumber) : base(fd, OutputNumber) {}
	}

	// --- PneuValve : Klasse fr ein Magnetventil -------------------------
	public class PneuValve : MonoOutput {
		public PneuValve(FishDevices fd, Out OutputNumber) 
			: this(fd, Convert.ToInt32(OutputNumber)) {}
		public PneuValve(FishDevices fd, int OutputNumber) : base(fd, OutputNumber) {}
	}
}
