// ------------------------------------------------------------------------------------
//
// ParkhausLinear.CS                                             Ulrich Mller 22.06.05
//                                                               PCS1700   C# Final SP2
// Modell der Parkhausschranke des Computing Starter Kits
// Schranke ffnet auf Anforderung eines Tasters und schliet nach Durchfahrt durch
// eine Lichtschranke. Dabei werden eine rote und eine grne Signallampe geschaltet
// 
// Programmierung erfolgte linear
// Einsatz : FishDevices40.DLL / umFish40.DLL
// 
// ------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using FishDevices40;

namespace ParkhausLinear
{
	public class frmMain : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Label lblStatus;
		private System.Windows.Forms.Button cmdAction;
		private System.ComponentModel.Container components = null;

		// --- RoboFace : Klassendeklarationen -----------------------------

		private FishDevices     fd;
		private LimitedMotor    Schranke;
		private LightBarrier    Durchfahrt;
		private Sensor          Anforderung;
		private DLamp           Rot;
		private DLamp           Gruen;

		public frmMain()
		{
			InitializeComponent();

			// --- RoboFace : Instanzierung, Ereignisse -----------------------

			fd          = new FishDevices();
			Schranke    = new LimitedMotor(fd, false, new Motor(fd, 3), 
                                         new Sensor(fd, 5), new Sensor(fd, 7));
			Durchfahrt  = new LightBarrier(fd, false, new DLamp(fd, 4), 
				                                 new PhotoTransistor(fd, 2));
			Anforderung = new Sensor(fd, 1);
			Rot         = new DLamp(fd, 2);
			Gruen       = new DLamp(fd, 1);

		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent()
		{
			this.lblStatus = new System.Windows.Forms.Label();
			this.cmdAction = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// lblStatus
			// 
			this.lblStatus.Font = new System.Drawing.Font("Tahoma", 11.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lblStatus.Location = new System.Drawing.Point(24, 16);
			this.lblStatus.Name = "lblStatus";
			this.lblStatus.Size = new System.Drawing.Size(368, 40);
			this.lblStatus.TabIndex = 0;
			this.lblStatus.Text = "START drcken";
			this.lblStatus.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// cmdAction
			// 
			this.cmdAction.Location = new System.Drawing.Point(24, 88);
			this.cmdAction.Name = "cmdAction";
			this.cmdAction.Size = new System.Drawing.Size(368, 40);
			this.cmdAction.TabIndex = 1;
			this.cmdAction.Text = "&START";
			this.cmdAction.Click += new System.EventHandler(this.cmdAction_Click);
			// 
			// frmMain
			// 
			this.AcceptButton = this.cmdAction;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.ClientSize = new System.Drawing.Size(416, 157);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.cmdAction,
																		  this.lblStatus});
			this.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.MaximizeBox = false;
			this.Name = "frmMain";
			this.Text = "FishDevices40 : Parkhaus - Linear";
			this.ResumeLayout(false);

		}
		#endregion

		[STAThread]
		static void Main() 
		{
			Application.Run(new frmMain());
		}

		private void cmdAction_Click(object sender, System.EventArgs e) {
			try {
				fd.Connect(IFTypen.ftIntelligent_IF, Port.COM2, 0);
				fd.Start();
				cmdAction.Enabled = false;

				Durchfahrt.On();
				Rot.On();

				lblStatus.Text = "--- Schranke betriebsbereit ---";

				do {
					Schranke.GoLeft();
					Schranke.WaitForDone();
					Anforderung.WaitForHigh();
					Schranke.GoRight();
					Schranke.WaitForDone();
					Rot.Off();
					Gruen.On();
					Durchfahrt.WaitForPassed();
					fd.Pause(555);
					Rot.On();
					Gruen.Off();
				} while(!fd.Finish());
				fd.DisConnect();
				this.Close();
			}
			catch (FishDevException efd) {
				lblStatus.Text = efd.Message;
			}
		}
	}
}
