// ------------------------------------------------------------------------------------
//
// ParkhausLinear.CS                                             Ulrich Mller 14.10.04
//                                                               PCS1700   C# Final SP2
// Modell der Parkhausschranke des Computing Starter Kits
// Schranke ffnet auf Anforderung eines Tasters und schliet nach Durchfahrt durch
// eine Lichtschranke. Dabei werden eine rote und eine grne Signallampe geschaltet
// 
// Programmierung erfolgte linear
// 
// ------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using RoboFa30;

namespace ParkhausLinear
{
	public class frmMain : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Label lblStatus;
		private System.Windows.Forms.Button cmdAction;
		private System.ComponentModel.Container components = null;

		// --- RoboFace : Klassendeklarationen -----------------------------

		private RoboFace        ft;
		private ParkSchranke    Schranke;
		private LightBarrier    Durchfahrt;
		private Taster          Anforderung;
		private DLamp           Rot;
		private DLamp           Gruen;

		public frmMain()
		{
			InitializeComponent();

			// --- RoboFace : Instanzierung, Ereignisse -----------------------

			ft          = new RoboFace();
			Schranke    = new ParkSchranke(ft, new Motor(ft, 3), 
                                     new Sensor(ft, 5), new Sensor(ft, 7));
			Durchfahrt  = new LightBarrier(ft, new DLamp(ft, 4), 
				                                 new PhotoTransistor(ft, 2));
			Anforderung = new Taster(ft, 1);
			Rot         = new DLamp(ft, 2);
			Gruen       = new DLamp(ft, 1);

		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent()
		{
			this.lblStatus = new System.Windows.Forms.Label();
			this.cmdAction = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// lblStatus
			// 
			this.lblStatus.Font = new System.Drawing.Font("Tahoma", 11.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lblStatus.Location = new System.Drawing.Point(24, 16);
			this.lblStatus.Name = "lblStatus";
			this.lblStatus.Size = new System.Drawing.Size(368, 40);
			this.lblStatus.TabIndex = 0;
			this.lblStatus.Text = "START drcken";
			this.lblStatus.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// cmdAction
			// 
			this.cmdAction.Location = new System.Drawing.Point(24, 88);
			this.cmdAction.Name = "cmdAction";
			this.cmdAction.Size = new System.Drawing.Size(368, 40);
			this.cmdAction.TabIndex = 1;
			this.cmdAction.Text = "&START";
			this.cmdAction.Click += new System.EventHandler(this.cmdAction_Click);
			// 
			// frmMain
			// 
			this.AcceptButton = this.cmdAction;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.ClientSize = new System.Drawing.Size(416, 157);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																																	this.cmdAction,
																																	this.lblStatus});
			this.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.MaximizeBox = false;
			this.Name = "frmMain";
			this.Text = "RoboFace : Parkhaus - Linear";
			this.ResumeLayout(false);

		}
		#endregion

		[STAThread]
		static void Main() 
		{
			Application.Run(new frmMain());
		}

		private void cmdAction_Click(object sender, System.EventArgs e) {
			try {
				ft.ConnectRobo(2);
				ft.StartRobo();
				cmdAction.Enabled = false;

				Durchfahrt.On();
				Rot.On();

				lblStatus.Text = "--- Schranke betriebsbereit ---";

				do {
					Schranke.Schliessen();
					Anforderung.Oeffnen();
					Schranke.Oeffnen();
					Rot.Off();
					Gruen.On();
					Durchfahrt.WaitForPassed();
					Rot.On();
					Gruen.Off();
				} while(!ft.Finish());

				lblStatus.Text = "--- Schrankenbetrieb wird beendet ---";
				ft.Pause(1234);
				ft.StopRobo();
				ft.DisconnectRobo();
				this.Close();
			}
			catch (RoboFaceException eft) {
				lblStatus.Text = eft.Message;
			}
		}
	}

	public class ParkSchranke : LimitedMotor {
		public ParkSchranke(RoboFace ft, Motor Mot, Sensor ZuSwitch, Sensor AufSwitch)
			: base(ft, Mot, ZuSwitch, AufSwitch){
		}
		public void Oeffnen() {
			base.GoRight();
			base.WaitForDone();
		}
		public void Schliessen() {
			base.GoLeft();
			base.WaitForDone();
		}
	}
	public class Taster : BinaryInput {
		public Taster(RoboFace ft, int ENr) : base(ft, ENr) {
		}
		public void Oeffnen() {
			base.WaitForHigh();
		}
	}
}
