// --------------------------------------------------------------------------------------
//
// VoltageInput.CS                                                 Ulrich Mller 19.06.05
//                                                                 PCS1700   C# Final SP2
// Spannungseingnge A1, A2, AV
//
// --------------------------------------------------------------------------------------

using System;
using System.Threading;

namespace FishDevices40 {
	public class VoltageInput : DeviceBase {
		private int limitLow, limitHigh;
		private int actValue, oldRange;

		public delegate void Limit(object sender, int Value);
		public event         Limit ChangedToLow;
		public event         Limit ChangedToHigh;
		public event         Limit ChangedToNormal;
		public event         Limit ShowActualValue;

        // --- Constructor -------------------------------------------------------------

		public VoltageInput(FishDevices fd, Inp VoltNumber, 
			double LimitLow, double LimitHigh) 
			: this(fd, true, Convert.ToInt32(VoltNumber), LimitLow, LimitHigh) {}
		public VoltageInput(FishDevices fd, int VoltNumber, 
			double LimitLow, double LimitHigh) 
			: this(fd, true, VoltNumber, LimitLow, LimitHigh) {}
		public VoltageInput(FishDevices fd, bool WithEvents, 
			Inp VoltNumber, double LimitLow, double LimitHigh) 
			: this(fd, Convert.ToInt32(VoltNumber), LimitLow, LimitHigh){}
		public VoltageInput(FishDevices fd, bool WithEvents, 
			int VoltNumber, double LimitLow, double LimitHigh) 
			: base(fd, WithEvents, VoltNumber) {
			this.limitLow  = (int)(LimitLow*100.0);
			this.limitHigh = (int)(LimitHigh*100.0);
			this.oldRange = 0;
			base.threadInterval = 22;
			fd.AddDevice(this);
			if(base.WithEvents) {
				base.deviceThread = new Thread(new ThreadStart(EventLoop));
				base.deviceThread.IsBackground = true;
			}
		}

		// --- VoltageLoop : Scannen des Volt-Einganges und Auslsen von Ereignissen ---

		private void EventLoop() {
			while(!fd.NotHalt && !fd.ESC) {
				actValue = fd.GetVoltage(base.DeviceNumber);
				if((uint)actValue == FishBase.rbFehler)
					throw(new FishDevException("InterfaceProblem.VoltageInput"));
				if(actValue < limitLow && oldRange != 1) {
					oldRange = 1;
					if(ChangedToLow != null) ChangedToLow(this, actValue);
				}
				else if(actValue > limitHigh && oldRange != 3) {
					oldRange = 3;
					if(ChangedToHigh != null) ChangedToHigh(this, actValue);
				}
				else if(actValue >= limitLow && actValue <= limitHigh 
					&& oldRange != 2) {
					oldRange = 2;
					if(ChangedToNormal != null) ChangedToNormal(this, actValue);
				}
				if(ShowActualValue != null) ShowActualValue(this, actValue);
				Thread.Sleep(base.threadInterval);
			}
		}
		// --- Eigenschaften des Volt-Einganges -----------------------------

		public int VoltNumber {
			get{return base.DeviceNumber;}
		}
		public double ActualValue {
			get {
				double res;
				try {res = ((double)fd.GetVoltage(base.DeviceNumber))/100.00;
					if(base.DeviceNumber == 3) res *= 3.0;
					return res;}
				catch(FishFaceException eft){
					throw(new FishDevException(eft.Message + ".VoltageInput.ActualValue"));
				}
			}
		}
		public double LimitHigh {
			get{return ((double)(this.limitHigh)/100.0);}
			set{limitHigh = (int)(value*100.0);}
		}
		public double LimitLow {
			get{return ((double)(this.limitLow)/100.0);}
			set{limitLow = (int)(value*100.0);}
		}
	}
}
